@@ -149,8 +149,6 @@ class CCodeGenerator:
             metadata.String: self._generate_serialize_string,
         }
         self._saved_byte_offsets = {}
-        self._uf_written = False
-        self._ud_written = False
         self._sasa = _StaticAlignSizeAutomatonByteOffset()
 
     def _generate_ctx_parent(self):
@@ -522,7 +520,8 @@ class CCodeGenerator:
         self._reset_per_func_state()
         self._generate_func_get_event_size_proto(stream, event)
         tmpl = templates._FUNC_GET_EVENT_SIZE_BODY_BEGIN
-        self._cg.add_lines(tmpl)
+        lines = tmpl.format(prefix=self._cfg.prefix)
+        self._cg.add_lines(lines)
         self._cg.add_empty_line()
         self._cg.indent()
         func = self._generate_func_get_event_size_from_entity
@@ -566,50 +565,54 @@ class CCodeGenerator:
         tmpl = templates._FUNC_SERIALIZE_EVENT_PROTO_END
         self._cg.add_lines(tmpl)
 
-    def _generate_bitfield_write(self, var, ctx, t):
+    def _generate_bitfield_write(self, ctype, var, ctx, t):
         ptr = '&{ctx}->buf[_BITS_TO_BYTES({ctx}->at)]'.format(ctx=ctx)
         start = self._sasa.byte_offset
         suffix = 'le' if t.byte_order is metadata.ByteOrder.LE else 'be'
         func = '{}bt_bitfield_write_{}'.format(self._cfg.prefix, suffix)
-        call = '{}({}, uint8_t, {}, {}, {});'.format(func, ptr, start, t.size,
-                                                     var)
+        call_fmt = '{func}({ptr}, uint8_t, {start}, {size}, {ctype}, ({ctype}) {var});'
+        call = call_fmt.format(func=func, ptr=ptr, start=start, size=t.size,
+                               ctype=ctype, var=var)
         self._cg.add_line(call)
 
     def _generate_serialize_int(self, var, ctx, t):
-        self._generate_bitfield_write(var, ctx, t)
+        ctype = self._get_int_ctype(t)
+        self._generate_bitfield_write(ctype, var, ctx, t)
         self._generate_incr_pos('{}->at'.format(ctx), t.size)
 
     def _generate_serialize_float(self, var, ctx, t):
         ctype = self._get_type_ctype(t)
+        flt_dbl = False
 
         if ctype == 'float' or ctype == 'double':
-            gen_union_var = False
+            flt_dbl = True
 
             if ctype == 'float':
-                if not self._uf_written:
-                    self._uf_written = True
-                    gen_union_var = True
-
                 union_name = 'f2u'
+                int_ctype = 'uint32_t'
             elif ctype == 'double':
-                if not self._ud_written:
-                    self._ud_written = True
-                    gen_union_var = True
-
                 union_name = 'd2u'
+                int_ctype = 'uint64_t'
 
-            if gen_union_var:
-                # union for reading the bytes of the floating point number
-
-                self._cg.add_line('union {name} {name};'.format(name=union_name))
-                self._cg.add_empty_line()
-
+            # union for reading the bytes of the floating point number
+            self._cg.add_empty_line()
+            self._cg.add_line('{')
+            self._cg.indent()
+            self._cg.add_line('union {name} {name};'.format(name=union_name))
+            self._cg.add_empty_line()
             self._cg.add_line('{}.f = {};'.format(union_name, var))
             bf_var = '{}.u'.format(union_name)
         else:
             bf_var = '({}) {}'.format(ctype, var)
+            int_ctype = ctype
+
+        self._generate_bitfield_write(int_ctype, bf_var, ctx, t)
+
+        if flt_dbl:
+            self._cg.unindent()
+            self._cg.add_line('}')
+            self._cg.add_empty_line()
 
-        self._generate_bitfield_write(bf_var, ctx, t)
         self._generate_incr_pos('{}->at'.format(ctx), t.size)
 
     def _generate_serialize_enum(self, var, ctx, t):
@@ -651,7 +654,8 @@ class CCodeGenerator:
         self._reset_per_func_state()
         self._generate_func_serialize_event_proto(stream, event)
         tmpl = templates._FUNC_SERIALIZE_EVENT_BODY_BEGIN
-        self._cg.add_lines(tmpl)
+        lines = tmpl.format(prefix=self._cfg.prefix)
+        self._cg.add_lines(lines)
         self._cg.indent()
 
         if stream.event_header_type is not None:
@@ -720,7 +724,8 @@ class CCodeGenerator:
         self._reset_per_func_state()
         self._generate_func_serialize_stream_event_header_proto(stream)
         tmpl = templates._FUNC_SERIALIZE_STREAM_EVENT_HEADER_BODY_BEGIN
-        self._cg.add_lines(tmpl)
+        lines = tmpl.format(prefix=self._cfg.prefix)
+        self._cg.add_lines(lines)
         self._cg.indent()
 
         if stream.event_header_type is not None:
@@ -757,7 +762,8 @@ class CCodeGenerator:
         self._reset_per_func_state()
         self._generate_func_serialize_stream_event_context_proto(stream)
         tmpl = templates._FUNC_SERIALIZE_STREAM_EVENT_CONTEXT_BODY_BEGIN
-        self._cg.add_lines(tmpl)
+        lines = tmpl.format(prefix=self._cfg.prefix)
+        self._cg.add_lines(lines)
         self._cg.indent()
         func = self._generate_func_serialize_event_from_entity
 
@@ -792,8 +798,7 @@ class CCodeGenerator:
         self._sasa.byte_offset = self._saved_byte_offsets[name]
 
     def _reset_per_func_state(self):
-        self._uf_written = False
-        self._ud_written = False
+        pass
 
     def _generate_func_open(self, stream):
         def generate_save_offset(name):
