@@ -84,7 +84,7 @@ _CTX_END = '};'
 
 _FUNC_INIT_PROTO = '''/* initialize context */
 void {prefix}init(
-	void *ctx,
+	void *vctx,
 	uint8_t *buf,
 	uint32_t buf_size,
 	struct {prefix}platform_callbacks cbs,
@@ -93,14 +93,14 @@ void {prefix}init(
 
 
 _FUNC_INIT_BODY = '''{{
-	struct {prefix}ctx *{prefix}ctx = ctx;
-	{prefix}ctx->cbs = cbs;
-	{prefix}ctx->data = data;
-	{prefix}ctx->buf = buf;
-	{prefix}ctx->packet_size = _BYTES_TO_BITS(buf_size);
-	{prefix}ctx->at = 0;
-	{prefix}ctx->events_discarded = 0;
-	{prefix}ctx->packet_is_open = 0;
+	struct {prefix}ctx *ctx = FROM_VOID_PTR(struct {prefix}ctx, vctx);
+	ctx->cbs = cbs;
+	ctx->data = data;
+	ctx->buf = buf;
+	ctx->packet_size = _BYTES_TO_BITS(buf_size);
+	ctx->at = 0;
+	ctx->events_discarded = 0;
+	ctx->packet_is_open = 0;
 }}'''
 
 
@@ -154,30 +154,31 @@ _FUNC_TRACE_BODY = '''{{
 	uint32_t ev_size;
 
 	/* get event size */
-	ev_size = _get_event_size_{sname}_{evname}((void *) ctx{params});
+	ev_size = _get_event_size_{sname}_{evname}(TO_VOID_PTR(ctx){params});
 
 	/* do we have enough space to serialize? */
-	if (!_reserve_event_space((void *) ctx, ev_size)) {{
+	if (!_reserve_event_space(TO_VOID_PTR(ctx), ev_size)) {{
 		/* no: forget this */
 		return;
 	}}
 
 	/* serialize event */
-	_serialize_event_{sname}_{evname}((void *) ctx{params});
+	_serialize_event_{sname}_{evname}(TO_VOID_PTR(ctx){params});
 
 	/* commit event */
-	_commit_event((void *) ctx);
+	_commit_event(TO_VOID_PTR(ctx));
 }}'''
 
 
 _FUNC_GET_EVENT_SIZE_PROTO_BEGIN = '''static uint32_t _get_event_size_{sname}_{evname}(
-	struct {prefix}ctx *ctx'''
+	void *vctx'''
 
 
 _FUNC_GET_EVENT_SIZE_PROTO_END = ')'
 
 
-_FUNC_GET_EVENT_SIZE_BODY_BEGIN = '''{
+_FUNC_GET_EVENT_SIZE_BODY_BEGIN = '''{{
+	struct {prefix}ctx *ctx = FROM_VOID_PTR(struct {prefix}ctx, vctx);
 	uint32_t at = ctx->at;'''
 
 
@@ -186,40 +187,43 @@ _FUNC_GET_EVENT_SIZE_BODY_END = '''	return at - ctx->at;
 
 
 _FUNC_SERIALIZE_STREAM_EVENT_HEADER_PROTO_BEGIN = '''static void _serialize_stream_event_header_{sname}(
-	struct {prefix}ctx *ctx,
+	void *vctx,
 	uint32_t event_id'''
 
 
 _FUNC_SERIALIZE_STREAM_EVENT_HEADER_PROTO_END = ')'
 
 
-_FUNC_SERIALIZE_STREAM_EVENT_HEADER_BODY_BEGIN = '{'
+_FUNC_SERIALIZE_STREAM_EVENT_HEADER_BODY_BEGIN = '''{{
+	struct {prefix}ctx *ctx = FROM_VOID_PTR(struct {prefix}ctx, vctx);'''
 
 
 _FUNC_SERIALIZE_STREAM_EVENT_HEADER_BODY_END = '}'
 
 
 _FUNC_SERIALIZE_STREAM_EVENT_CONTEXT_PROTO_BEGIN = '''static void _serialize_stream_event_context_{sname}(
-	struct {prefix}ctx *ctx'''
+	void *vctx'''
 
 
 _FUNC_SERIALIZE_STREAM_EVENT_CONTEXT_PROTO_END = ')'
 
 
-_FUNC_SERIALIZE_STREAM_EVENT_CONTEXT_BODY_BEGIN = '{'
+_FUNC_SERIALIZE_STREAM_EVENT_CONTEXT_BODY_BEGIN = '''{{
+	struct {prefix}ctx *ctx = FROM_VOID_PTR(struct {prefix}ctx, vctx);'''
 
 
 _FUNC_SERIALIZE_STREAM_EVENT_CONTEXT_BODY_END = '}'
 
 
 _FUNC_SERIALIZE_EVENT_PROTO_BEGIN = '''static void _serialize_event_{sname}_{evname}(
-	struct {prefix}ctx *ctx'''
+	void *vctx'''
 
 
 _FUNC_SERIALIZE_EVENT_PROTO_END = ')'
 
 
-_FUNC_SERIALIZE_EVENT_BODY_BEGIN = '{'
+_FUNC_SERIALIZE_EVENT_BODY_BEGIN = '''{{
+	struct {prefix}ctx *ctx = FROM_VOID_PTR(struct {prefix}ctx, vctx);'''
 
 
 _FUNC_SERIALIZE_EVENT_BODY_END = '}'
@@ -229,16 +233,44 @@ _HEADER_BEGIN = '''#ifndef _{ucprefix}H
 #define _{ucprefix}H
 
 /*
+ * The MIT License (MIT)
+ *
+ * Copyright (c) 2015-2016 Philippe Proulx <pproulx@efficios.com>
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+ * THE SOFTWARE.
+ *
+ * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+ *
  * The following C code was generated by barectf {version}
  * on {date}.
  *
- * For more details, see <https://github.com/efficios/barectf>.
+ * For more details, see <http://barectf.org>.
  */
 
 #include <stdint.h>
 
 #include "{bitfield_header_filename}"
 
+#ifdef __cplusplus
+extern "C" {{
+#endif
+
 {prefix_def}
 {default_stream_def}
 
@@ -256,14 +288,43 @@ uint32_t {prefix}packet_buf_size(void *ctx);
 int {prefix}packet_is_open(void *ctx);'''
 
 
-_HEADER_END = '#endif /* _{ucprefix}H */'
+_HEADER_END = '''#ifdef __cplusplus
+}}
+#endif
+
+#endif /* _{ucprefix}H */
+'''
 
 
 _C_SRC = '''/*
+ * The MIT License (MIT)
+ *
+ * Copyright (c) 2015-2016 Philippe Proulx <pproulx@efficios.com>
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+ * THE SOFTWARE.
+ *
+ * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+ *
  * The following C code was generated by barectf {version}
  * on {date}.
  *
- * For more details, see <https://github.com/efficios/barectf>.
+ * For more details, see <http://barectf.org>.
  */
 
 #include <stdint.h>
@@ -277,6 +338,14 @@ _C_SRC = '''/*
 		(_at) = ((_at) + ((_align) - 1)) & -(_align);	\\
 	}} while (0)
 
+#ifdef __cplusplus
+# define TO_VOID_PTR(_value)		static_cast<void *>(_value)
+# define FROM_VOID_PTR(_type, _value)	static_cast<_type *>(_value)
+#else
+# define TO_VOID_PTR(_value)		((void *) (_value))
+# define FROM_VOID_PTR(_type, _value)	((_type *) (_value))
+#endif
+
 #define _BITS_TO_BYTES(_x)	((_x) >> 3)
 #define _BYTES_TO_BITS(_x)	((_x) << 3)
 
@@ -292,49 +361,51 @@ union d2u {{
 
 uint32_t {prefix}packet_size(void *ctx)
 {{
-	return ((struct {prefix}ctx *) ctx)->packet_size;
+	return FROM_VOID_PTR(struct {prefix}ctx, ctx)->packet_size;
 }}
 
 int {prefix}packet_is_full(void *ctx)
 {{
-	struct {prefix}ctx *cctx = ctx;
+	struct {prefix}ctx *cctx = FROM_VOID_PTR(struct {prefix}ctx, ctx);
 
 	return cctx->at == cctx->packet_size;
 }}
 
 int {prefix}packet_is_empty(void *ctx)
 {{
-	struct {prefix}ctx *cctx = ctx;
+	struct {prefix}ctx *cctx = FROM_VOID_PTR(struct {prefix}ctx, ctx);
 
 	return cctx->at <= cctx->off_content;
 }}
 
 uint32_t {prefix}packet_events_discarded(void *ctx)
 {{
-	return ((struct {prefix}ctx *) ctx)->events_discarded;
+	return FROM_VOID_PTR(struct {prefix}ctx, ctx)->events_discarded;
 }}
 
 uint8_t *{prefix}packet_buf(void *ctx)
 {{
-	return ((struct {prefix}ctx *) ctx)->buf;
+	return FROM_VOID_PTR(struct {prefix}ctx, ctx)->buf;
 }}
 
 uint32_t {prefix}packet_buf_size(void *ctx)
 {{
-	return _BITS_TO_BYTES(((struct {prefix}ctx *) ctx)->packet_size);
+	struct {prefix}ctx *cctx = FROM_VOID_PTR(struct {prefix}ctx, ctx);
+
+	return _BITS_TO_BYTES(cctx->packet_size);
 }}
 
 void {prefix}packet_set_buf(void *ctx, uint8_t *buf, uint32_t buf_size)
 {{
-	struct {prefix}ctx *{prefix}ctx = ctx;
+	struct {prefix}ctx *cctx = FROM_VOID_PTR(struct {prefix}ctx, ctx);
 
-	{prefix}ctx->buf = buf;
-	{prefix}ctx->packet_size = _BYTES_TO_BITS(buf_size);
+	cctx->buf = buf;
+	cctx->packet_size = _BYTES_TO_BITS(buf_size);
 }}
 
 int {prefix}packet_is_open(void *ctx)
 {{
-	return ((struct {prefix}ctx *) ctx)->packet_is_open;
+	return FROM_VOID_PTR(struct {prefix}ctx, ctx)->packet_is_open;
 }}
 
 static
@@ -346,15 +417,11 @@ void _write_cstring(struct {prefix}ctx *ctx, const char *src)
 	ctx->at += _BYTES_TO_BITS(sz);
 }}
 
-static inline
-int _packet_is_full(struct {prefix}ctx *ctx)
-{{
-	return {prefix}packet_is_full(ctx);
-}}
-
 static
-int _reserve_event_space(struct {prefix}ctx *ctx, uint32_t ev_size)
+int _reserve_event_space(void *vctx, uint32_t ev_size)
 {{
+	struct {prefix}ctx *ctx = FROM_VOID_PTR(struct {prefix}ctx, vctx);
+
 	/* event _cannot_ fit? */
 	if (ev_size > (ctx->packet_size - ctx->off_content)) {{
 		ctx->events_discarded++;
@@ -398,8 +465,10 @@ int _reserve_event_space(struct {prefix}ctx *ctx, uint32_t ev_size)
 }}
 
 static
-void _commit_event(struct {prefix}ctx *ctx)
+void _commit_event(void *vctx)
 {{
+	struct {prefix}ctx *ctx = FROM_VOID_PTR(struct {prefix}ctx, vctx);
+
 	/* is packet full? */
 	if ({prefix}packet_is_full(ctx)) {{
 		/* yes: close it now */
@@ -437,44 +506,28 @@ _BITFIELD = '''#ifndef _$PREFIX$BITFIELD_H
  * SOFTWARE.
  */
 
-#include <stdint.h>	/* C99 5.2.4.2 Numerical limits */
 #include <limits.h>
 
+#ifdef __cplusplus
+# define CAST_PTR(_type, _value) \\
+	static_cast<_type>(static_cast<void *>(_value))
+#else
+# define CAST_PTR(_type, _value)	((void *) (_value))
+#endif
+
 #define $PREFIX$BYTE_ORDER $ENDIAN_DEF$
 
 /* We can't shift a int from 32 bit, >> 32 and << 32 on int is undefined */
-#define _$prefix$bt_piecewise_rshift(_v, _shift) \\
-__extension__ ({									\\
-	__typeof__(_v) ___v = (_v);					\\
-	__typeof__(_shift) ___shift = (_shift);				\\
-	unsigned long sb = (___shift) / (sizeof(___v) * CHAR_BIT - 1);	\\
-	unsigned long final = (___shift) % (sizeof(___v) * CHAR_BIT - 1); \\
+#define _$prefix$bt_piecewise_rshift(_vtype, _v, _shift) \\
+do {									\\
+	unsigned long ___shift = (_shift);				\\
+	unsigned long sb = (___shift) / (sizeof(_v) * CHAR_BIT - 1);	\\
+	unsigned long final = (___shift) % (sizeof(_v) * CHAR_BIT - 1); \\
 									\\
 	for (; sb; sb--)						\\
-		___v >>= sizeof(___v) * CHAR_BIT - 1;			\\
-	___v >>= final;							\\
-})
-
-#define _$prefix$bt_piecewise_lshift(_v, _shift) \\
-__extension__ ({									\\
-	__typeof__(_v) ___v = (_v);					\\
-	__typeof__(_shift) ___shift = (_shift);				\\
-	unsigned long sb = (___shift) / (sizeof(___v) * CHAR_BIT - 1);	\\
-	unsigned long final = (___shift) % (sizeof(___v) * CHAR_BIT - 1); \\
-									\\
-	for (; sb; sb--)						\\
-		___v <<= sizeof(___v) * CHAR_BIT - 1;			\\
-	___v <<= final;							\\
-})
-
-#define _$prefix$bt_is_signed_type(type)	((type) -1 < (type) 0)
-
-#define _$prefix$bt_unsigned_cast(type, v) \\
-__extension__ ({									\\
-	(sizeof(v) < sizeof(type)) ?					\\
-		((type) (v)) & (~(~(type) 0 << (sizeof(v) * CHAR_BIT))) : \\
-		(type) (v);						\\
-})
+		_v >>= sizeof(_v) * CHAR_BIT - 1;			\\
+	_v >>= final;							\\
+} while (0)
 
 /*
  * $prefix$bt_bitfield_write - write integer to a bitfield in native endianness
@@ -494,10 +547,10 @@ __extension__ ({									\\
  * Also, consecutive bitfields are placed from higher to lower bits.
  */
 
-#define _$prefix$bt_bitfield_write_le(_ptr, type, _start, _length, _v) \\
+#define _$prefix$bt_bitfield_write_le(_ptr, type, _start, _length, _vtype, _v) \\
 do {									\\
-	__typeof__(_v) __v = (_v);					\\
-	type *__ptr = (void *) (_ptr);					\\
+	_vtype __v = (_v);					\\
+	type *__ptr = CAST_PTR(type *, _ptr);				\\
 	unsigned long __start = (_start), __length = (_length);		\\
 	type mask, cmask;						\\
 	unsigned long ts = sizeof(type) * CHAR_BIT; /* type size */	\\
@@ -513,7 +566,7 @@ do {									\\
 									\\
 	/* Trim v high bits */						\\
 	if (__length < sizeof(__v) * CHAR_BIT)				\\
-		__v &= ~((~(__typeof__(__v)) 0) << __length);		\\
+		__v &= ~((~(_vtype) 0) << __length);		\\
 									\\
 	/* We can now append v with a simple "or", shift it piece-wise */ \\
 	this_unit = start_unit;						\\
@@ -534,13 +587,13 @@ do {									\\
 		cmask &= ~mask;						\\
 		__ptr[this_unit] &= mask;				\\
 		__ptr[this_unit] |= cmask;				\\
-		__v = _$prefix$bt_piecewise_rshift(__v, ts - cshift); \\
+		_$prefix$bt_piecewise_rshift(_vtype, __v, ts - cshift); \\
 		__start += ts - cshift;					\\
 		this_unit++;						\\
 	}								\\
 	for (; this_unit < end_unit - 1; this_unit++) {			\\
 		__ptr[this_unit] = (type) __v;				\\
-		__v = _$prefix$bt_piecewise_rshift(__v, ts); \\
+		_$prefix$bt_piecewise_rshift(_vtype, __v, ts); 		\\
 		__start += ts;						\\
 	}								\\
 	if (end % ts) {							\\
@@ -553,10 +606,10 @@ do {									\\
 		__ptr[this_unit] = (type) __v;				\\
 } while (0)
 
-#define _$prefix$bt_bitfield_write_be(_ptr, type, _start, _length, _v) \\
+#define _$prefix$bt_bitfield_write_be(_ptr, type, _start, _length, _vtype, _v) \\
 do {									\\
-	__typeof__(_v) __v = (_v);					\\
-	type *__ptr = (void *) (_ptr);					\\
+	_vtype __v = (_v);					\\
+	type *__ptr = CAST_PTR(type *, _ptr);				\\
 	unsigned long __start = (_start), __length = (_length);		\\
 	type mask, cmask;						\\
 	unsigned long ts = sizeof(type) * CHAR_BIT; /* type size */	\\
@@ -572,7 +625,7 @@ do {									\\
 									\\
 	/* Trim v high bits */						\\
 	if (__length < sizeof(__v) * CHAR_BIT)				\\
-		__v &= ~((~(__typeof__(__v)) 0) << __length);		\\
+		__v &= ~((~(_vtype) 0) << __length);			\\
 									\\
 	/* We can now append v with a simple "or", shift it piece-wise */ \\
 	this_unit = end_unit - 1;					\\
@@ -593,13 +646,13 @@ do {									\\
 		cmask &= ~mask;						\\
 		__ptr[this_unit] &= mask;				\\
 		__ptr[this_unit] |= cmask;				\\
-		__v = _$prefix$bt_piecewise_rshift(__v, cshift); \\
+		_$prefix$bt_piecewise_rshift(__v, cshift); \\
 		end -= cshift;						\\
 		this_unit--;						\\
 	}								\\
 	for (; (long) this_unit >= (long) start_unit + 1; this_unit--) { \\
 		__ptr[this_unit] = (type) __v;				\\
-		__v = _$prefix$bt_piecewise_rshift(__v, ts); \\
+		_$prefix$bt_piecewise_rshift(__v, ts); \\
 		end -= ts;						\\
 	}								\\
 	if (__start % ts) {						\\
@@ -619,19 +672,19 @@ do {									\\
 
 #if ($PREFIX$BYTE_ORDER == LITTLE_ENDIAN)
 
-#define $prefix$bt_bitfield_write_le(ptr, type, _start, _length, _v) \\
-	_$prefix$bt_bitfield_write_le(ptr, type, _start, _length, _v)
+#define $prefix$bt_bitfield_write_le(ptr, type, _start, _length, _vtype, _v) \\
+	_$prefix$bt_bitfield_write_le(ptr, type, _start, _length, _vtype, _v)
 
-#define $prefix$bt_bitfield_write_be(ptr, type, _start, _length, _v) \\
-	_$prefix$bt_bitfield_write_be(ptr, unsigned char, _start, _length, _v)
+#define $prefix$bt_bitfield_write_be(ptr, type, _start, _length, _vtype, _v) \\
+	_$prefix$bt_bitfield_write_be(ptr, unsigned char, _start, _length, _vtype, _v)
 
 #elif ($PREFIX$BYTE_ORDER == BIG_ENDIAN)
 
-#define $prefix$bt_bitfield_write_le(ptr, type, _start, _length, _v) \\
-	_$prefix$bt_bitfield_write_le(ptr, unsigned char, _start, _length, _v)
+#define $prefix$bt_bitfield_write_le(ptr, type, _start, _length, _vtype, _v) \\
+	_$prefix$bt_bitfield_write_le(ptr, unsigned char, _start, _length, _vtype, _v)
 
-#define $prefix$bt_bitfield_write_be(ptr, type, _start, _length, _v) \\
-	_$prefix$bt_bitfield_write_be(ptr, type, _start, _length, _v)
+#define $prefix$bt_bitfield_write_be(ptr, type, _start, _length, _vtype, _v) \\
+	_$prefix$bt_bitfield_write_be(ptr, type, _start, _length, _vtype, _v)
 
 #else /* ($PREFIX$BYTE_ORDER == PDP_ENDIAN) */
 
